@@ -1,9 +1,9 @@
-from functools import partial
+from functools import partial, wraps
 from . expr import Parser
 from . mqtt import topic
 
 
-__all__ = ('StateAwareMixin', 'when')
+__all__ = ('StateAwareMixin', 'toggle', 'when')
 
 
 class StateAwareMixin:
@@ -56,29 +56,52 @@ class StateAwareMixin:
     
     def invoke_handlers(self, topic, payload):
         'Run through conditions and invoke appropriate handlers'
-        for expr, action in self.conditions.items():
-            if topic not in expr.keys: continue
-            
-            id = action.func.__name__ if type(action) is partial else action.__name__
-            if expr.keys <= self.current_state.keys():
-                if expr(self.current_state):
-                    self.log.info('Invoking: %s', id)
-                    action(self)
-                else:
-                    self.log.debug('Skipping: %s', id)
+        for expr, method in self.conditions.items():
+            if topic in expr.keys and expr.keys <= self.current_state.keys():
+                method(self)
 
 
 def when(expression):
     ''' Decorator for status handler methods.
-        The decorated method is evaluated when it depends 
-        on the updated topic and when the expression evaluates
-        to `True`.
+        The decorated method is invoked when the expression depends 
+        on the updated topic and when it evaluates to `True`.
         For the expression grammar, see the docstrings in `expr.py`.
     '''
     
     predicate = StateAwareMixin.parser.parse(expression)
     
     def wrapper(method):
-        StateAwareMixin.conditions[predicate] = method
-        return method
+        id = method.func.__name__ if type(method) is partial else method.__name__
+        
+        @wraps(method)
+        def wrapped(self):
+            if predicate(self.current_state):
+                self.log.info('Invoking: %s', id)
+                method(self)
+            else:
+                self.log.debug('Skipping: %s', id)
+        StateAwareMixin.conditions[predicate] = wrapped
+        return wrapped
+    return wrapper
+
+
+def toggle(expression):
+    ''' Decorator for status handler methods.
+        The decorated method is invoked with the value of the expression
+        when the expression depends on the updated topic.
+        For the expression grammar, see the docstrings in `expr.py`.
+    '''
+    
+    predicate = StateAwareMixin.parser.parse(expression)
+    
+    def wrapper(method):
+        id = method.func.__name__ if type(method) is partial else method.__name__
+        
+        @wraps(method)
+        def wrapped(self):
+            condition = predicate(self.current_state)
+            self.log.info('Invoking: %s(%s)', id, condition)
+            method(self, condition)
+        StateAwareMixin.conditions[predicate] = wrapped
+        return wrapped
     return wrapper
